/** @file
    Declaration of all available decoders.
*/

#ifndef INCLUDE_RTL_433_DEVICES_H_
#define INCLUDE_RTL_433_DEVICES_H_

#include "r_device.h"

#define DEVICES \
    DECL(silvercrest) \
    DECL(rubicson) \
    DECL(prologue) \
    DECL(waveman) \
    DECL(new_template) \
    DECL(elv_em1000) \
    DECL(elv_ws2000) \
    DECL(lacrossetx) \
    DECL(new_template) \
    DECL(acurite_rain_896) \
    DECL(acurite_th) \
    DECL(oregon_scientific) \
    DECL(mebus433) \
    DECL(intertechno) \
    DECL(newkaku) \
    DECL(alectov1) \
    DECL(cardin) \
    DECL(fineoffset_WH2) \
    DECL(nexus) \
    DECL(ambient_weather) \
    DECL(calibeur_RF104) \
    DECL(X10_RF) \
    DECL(dsc_security) \
    DECL(brennenstuhl_rcs_2044) \
    DECL(gt_wt_02) \
    DECL(danfoss_CFR) \
    DECL(new_template) \
    DECL(new_template) \
    DECL(chuango) \
    DECL(generic_remote) \
    DECL(tfa_twin_plus_303049) \
    DECL(fineoffset_wh1080) \
    DECL(wt450) \
    DECL(lacrossews) \
    DECL(esperanza_ews) \
    DECL(efergy_e2_classic) \
    DECL(kw9015b) \
    DECL(generic_temperature_sensor) \
    DECL(wg_pb12v1) \
    DECL(acurite_txr) \
    DECL(acurite_986) \
    DECL(hideki_ts04) \
    DECL(oil_watchman) \
    DECL(current_cost) \
    DECL(emontx) \
    DECL(ht680) \
    DECL(s3318p) \
    DECL(akhan_100F14) \
    DECL(quhwa) \
    DECL(oregon_scientific_v1) \
    DECL(proove) \
    DECL(bresser_3ch) \
    DECL(springfield) \
    DECL(oregon_scientific_sl109h) \
    DECL(acurite_606) \
    DECL(tfa_pool_thermometer) \
    DECL(kedsum) \
    DECL(blyss) \
    DECL(steelmate) \
    DECL(schraeder) \
    DECL(lightwave_rf) \
    DECL(elro_db286a) \
    DECL(efergy_optical) \
    DECL(hondaremote) \
    DECL(new_template) \
    DECL(new_template) \
    DECL(radiohead_ask) \
    DECL(kerui) \
    DECL(fineoffset_wh1050) \
    DECL(honeywell) \
    DECL(maverick_et73x) \
    DECL(rftech) \
    DECL(lacrosse_tx141x) \
    DECL(acurite_00275rm) \
    DECL(lacrosse_tx35) \
    DECL(lacrosse_tx29) \
    DECL(vaillant_vrt340f) \
    DECL(fineoffset_WH25) \
    DECL(fineoffset_WH0530) \
    DECL(ibis_beacon) \
    DECL(oil_standard) \
    DECL(tpms_citroen) \
    DECL(oil_standard_ask) \
    DECL(thermopro_tp11) \
    DECL(solight_te44) \
    DECL(smoke_gs558) \
    DECL(generic_motion) \
    DECL(tpms_toyota) \
    DECL(tpms_ford) \
    DECL(tpms_renault) \
    DECL(infactory) \
    DECL(ft004b) \
    DECL(fordremote) \
    DECL(philips_aj3650) \
    DECL(schrader_EG53MA4) \
    DECL(nexa) \
    DECL(thermopro_tp12) \
    DECL(ge_coloreffects) \
    DECL(x10_sec) \
    DECL(interlogix) \
    DECL(dish_remote_6_3) \
    DECL(ss_sensor) \
    DECL(sensible_living) \
    DECL(m_bus_mode_c_t) \
    DECL(m_bus_mode_s) \
    DECL(m_bus_mode_r) \
    DECL(m_bus_mode_f) \
    DECL(wssensor) \
    DECL(wt1024) \
    DECL(tpms_pmv107j) \
    DECL(ttx201) \
    DECL(ambientweather_tx8300) \
    DECL(ambientweather_wh31e) \
    DECL(maverick_et73) \
    DECL(honeywell_wdb) \
    DECL(honeywell_wdb_fsk) \
    DECL(esa_energy) \
    DECL(bt_rain) \
    DECL(bresser_5in1) \
    DECL(digitech_xc0324) \
    DECL(opus_xt300) \
    DECL(fs20) \
    DECL(tpms_jansite) \
    DECL(lacrosse_ws7000) \
    DECL(ts_ft002) \
    DECL(companion_wtr001) \
    DECL(ecowitt) \
    DECL(directv) \
    DECL(eurochron) \
    DECL(ikea_sparsnas) \
    DECL(hcs200) \
    DECL(tfa_303196) \
    DECL(rubicson_48659) \
    DECL(holman_ws5029pcm) \
    DECL(philips_aj7010) \
    DECL(esic_emt7110) \
    DECL(gt_tmbbq05) \
    DECL(gt_wt_03) \
    DECL(norgo) \
    DECL(tpms_elantra2012) \
    DECL(auriol_hg02832) \
    DECL(fineoffset_WH51) \
    DECL(holman_ws5029pwm) \
    DECL(archos_tbh) \
    DECL(ws2032) \
    DECL(auriol_afw2a1) \
    DECL(tfa_drop_303233) \
    DECL(dsc_security_ws4945) \
    DECL(ert_scm) \
    DECL(klimalogg) \
    DECL(visonic_powercode) \
    DECL(eurochron_efth800) \
    DECL(cotech_36_7959) \
    DECL(scmplus) \
    DECL(fineoffset_wh1080_fsk) \
    DECL(tpms_abarth124) \
    DECL(missil_ml0757) \
    DECL(sharp_spc775) \
    DECL(insteon) \
    DECL(ert_idm) \
    DECL(ert_netidm) \
    DECL(thermopro_tx2) \
    DECL(acurite_590tx) \
    DECL(secplus_v2) \
    DECL(tfa_30_3221) \
    DECL(lacrosse_breezepro) \
    DECL(somfy_rts) \
    DECL(schrader_SMD3MA4) \
    DECL(nice_flor_s) \
    DECL(lacrosse_wr1) \
    DECL(lacrosse_th3) \
    DECL(bresser_6in1) \
    DECL(bresser_7in1) \
    DECL(ecodhome) \
    DECL(lacrosse_r1) \
    DECL(blueline) \
    DECL(burnhardbbq) \
    DECL(secplus_v1) \
    DECL(cavius) \
    DECL(tpms_jansite_solar) \
    DECL(abmt) \
    DECL(tfa_marbella) \
    DECL(auriol_ahfl) \
    DECL(auriol_aft77b2) \
    DECL(honeywell_cm921) \
    DECL(tpms_hyundai_vdo) \
    DECL(rojaflex) \
    DECL(marlec_solar) \
    DECL(somfy_iohc) \
    DECL(fineoffset_wh31l) \
    DECL(markisol) \
    DECL(govee) \
    DECL(cmr113) \
    DECL(inkbird_ith20r) \
    DECL(rainpoint) \
    DECL(atech_ws308) \
    DECL(acurite_01185m) \
    DECL(enocean_erp1) \
    DECL(megacode) \
    DECL(auriol_4ld5661) \
    DECL(tpms_truck) \
    DECL(funkbus_remote) \
    DECL(tpms_porsche) \
    DECL(jasco) \
    DECL(telldus_ft0385r) \
    DECL(lacrosse_tx34) \
    DECL(proflame2) \
    DECL(tpms_ave) \
    DECL(simplisafe_gen3) \
    DECL(yale_hsa) \
    DECL(regency_fan) \
    DECL(tpms_renault_0435r) \
    DECL(fineoffset_ws80) \
    DECL(emos_e6016) \
    DECL(emax) \
    DECL(ant_antplus) \
    DECL(emos_e6016_rain) \
    DECL(hcs200_fsk) \
    DECL(fineoffset_wh45) \
    DECL(maverick_xr30) \
    DECL(fineoffset_wn34) \
    DECL(rubicson_pool_48942) \
    DECL(badger_orion) \
    DECL(geo_minim) \
    DECL(tpms_tyreguard400) \
    DECL(tpms_kia) \
    DECL(srsmith_pool_srs_2c_tx) \
    DECL(neptune_r900) \
    DECL(wec2103) \
    DECL(vauno_en8822c) \
    DECL(govee_h5054) \
    DECL(tfa_14_1504_v2) \
    DECL(ced7000) \
    DECL(oil_watchman_advanced) \
    DECL(oil_smart) \
    DECL(gasmate_ba1008) \
    DECL(flowis) \
    DECL(m_bus_mode_c_t_downlink) \
    DECL(revolt_nc5462) \
    DECL(lacrosse_tx31u) \
    DECL(tpms_eezrv) \
    DECL(baldr_rain) \
    DECL(celsia_czc1) \
    DECL(fineoffset_ws90) \
    DECL(thermopro_tx2c) \
    DECL(tfa_303151) \
    DECL(bresser_leakage) \
    DECL(tpms_nissan) \
    DECL(bresser_lightning) \
    DECL(schou_72543_rain) \
    DECL(fineoffset_wh55) \
    DECL(tpms_bmw) \
    DECL(watts_thermostat) \
    DECL(thermor) \
    DECL(mueller_hotrod) \
    DECL(thermopro_tp28b) \
    DECL(tpms_bmwg3) \
    DECL(chamberlain_cwpirc) \
    DECL(thermopro_tp829b) \
    DECL(arad_ms_meter) \
    DECL(geevon) \
    DECL(fineoffset_wh46) \
    DECL(vevor_7in1) \
    DECL(arexx_ml) \
    DECL(rosstech_dcu706) \
    DECL(risco_agility) \
    DECL(thermopro_tp828b) \
    DECL(bresser_st1005h) \
    DECL(deltadore_x3d) \
    DECL(quinetic) \
    DECL(gridstream96) \
    DECL(gridstream192) \
    DECL(gridstream384) \
    DECL(revolt_zx7717) \
    DECL(tpms_gm) \
    DECL(rainpoint_hcs012arf) \
    DECL(apator_metra_erm30) \
    DECL(thermopro_tx7b) \
    DECL(nexus_sauna) \
    DECL(homelead_hg9901) \
    DECL(maverick_xr50) \
    DECL(orion_endpoint) \
    DECL(fineoffset_wh43) \
    DECL(baldr_therm) \
    DECL(bm5) \
    DECL(universalfanctrl) \
    DECL(fineoffset_ws85) \
    DECL(oria_wa150km) \
    DECL(ec3k) \
    DECL(orion_endpoint_2020) \

    /* Add new decoders here. */

#define DECL(name) extern r_device name;
DEVICES
#undef DECL

#endif /* INCLUDE_RTL_433_DEVICES_H_ */
